import os

import numpy as np
import pytest

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.processing.background import (
    LineSubtractionModule,
    MeanBackgroundSubtractionModule,
    NoddingBackgroundModule,
    SimpleBackgroundSubtractionModule,
)
from pynpoint.processing.pcabackground import DitheringBackgroundModule
from pynpoint.processing.stacksubset import StackCubesModule
from pynpoint.util.tests import (
    create_config,
    create_dither_data,
    create_star_data,
    remove_test_data,
)


class TestBackground:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_dither_data(self.test_dir + "dither")
        create_star_data(self.test_dir + "science")
        create_star_data(self.test_dir + "sky")
        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, folders=["dither", "science", "sky"])

    def test_read_data(self) -> None:

        module = FitsReadingModule(
            name_in="read1", image_tag="dither", input_dir=self.test_dir + "dither"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read1")

        data = self.pipeline.get_data("dither")
        assert np.sum(data) == pytest.approx(
            211.20534661914408, rel=self.limit, abs=0.0
        )
        assert data.shape == (20, 21, 21)

        module = FitsReadingModule(
            name_in="read2", image_tag="science", input_dir=self.test_dir + "science"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read2")

        data = self.pipeline.get_data("science")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

        module = FitsReadingModule(
            name_in="read3", image_tag="sky", input_dir=self.test_dir + "sky"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read3")

        data = self.pipeline.get_data("sky")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_simple_background(self) -> None:

        module = SimpleBackgroundSubtractionModule(
            shift=5, name_in="simple", image_in_tag="dither", image_out_tag="simple"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("simple")

        data = self.pipeline.get_data("simple")
        assert np.sum(data) == pytest.approx(0.0, rel=0.0, abs=1e-10)
        assert data.shape == (20, 21, 21)

    def test_mean_background_shift(self) -> None:

        module = MeanBackgroundSubtractionModule(
            shift=5,
            cubes=1,
            name_in="mean2",
            image_in_tag="dither",
            image_out_tag="mean2",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("mean2")

        data = self.pipeline.get_data("mean2")
        assert np.sum(data) == pytest.approx(2.473864361018551, rel=self.limit, abs=0.0)
        assert data.shape == (20, 21, 21)

    def test_mean_background_nframes(self) -> None:

        module = MeanBackgroundSubtractionModule(
            shift=None,
            cubes=1,
            name_in="mean1",
            image_in_tag="dither",
            image_out_tag="mean1",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("mean1")

        data = self.pipeline.get_data("mean1")
        assert np.sum(data) == pytest.approx(2.473864361018551, rel=self.limit, abs=0.0)
        assert data.shape == (20, 21, 21)

    def test_dithering_attributes(self) -> None:

        module = DitheringBackgroundModule(
            name_in="pca_dither1",
            image_in_tag="dither",
            image_out_tag="pca_dither1",
            center=None,
            cubes=None,
            size=0.2,
            gaussian=0.05,
            subframe=0.1,
            pca_number=1,
            mask_star=0.05,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("pca_dither1")

        data = self.pipeline.get_data("dither_dither_crop1")
        assert np.sum(data) == pytest.approx(54.62410860562912, rel=self.limit, abs=0.0)
        assert data.shape == (20, 9, 9)

        data = self.pipeline.get_data("dither_dither_star1")
        assert np.sum(data) == pytest.approx(
            54.873885838788595, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 9, 9)

        data = self.pipeline.get_data("dither_dither_mean1")
        assert np.sum(data) == pytest.approx(
            54.204960755115245, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 9, 9)

        data = self.pipeline.get_data("dither_dither_background1")
        assert np.sum(data) == pytest.approx(
            -0.24977723315947564, rel=self.limit, abs=0.0
        )
        assert data.shape == (15, 9, 9)

        data = self.pipeline.get_data("dither_dither_pca_fit1")
        assert np.sum(data) == pytest.approx(-0.6816458444287745, rel=1e-5, abs=0.0)
        assert data.shape == (5, 9, 9)

        data = self.pipeline.get_data("dither_dither_pca_res1")
        assert np.sum(data) == pytest.approx(55.63879076093719, rel=1e-6, abs=0.0)
        assert data.shape == (5, 9, 9)

        data = self.pipeline.get_data("dither_dither_pca_mask1")
        assert np.sum(data) == pytest.approx(360.0, rel=self.limit, abs=0.0)
        assert data.shape == (5, 9, 9)

        data = self.pipeline.get_data("pca_dither1")
        assert np.sum(data) == pytest.approx(208.24417329569593, rel=1e-6, abs=0.0)
        assert data.shape == (20, 9, 9)

        attr = self.pipeline.get_attribute(
            "dither_dither_pca_res1", "STAR_POSITION", static=False
        )
        assert np.sum(attr) == pytest.approx(40.0, rel=self.limit, abs=0.0)
        assert attr.shape == (5, 2)

    def test_dithering_center(self) -> None:

        module = DitheringBackgroundModule(
            name_in="pca_dither2",
            image_in_tag="dither",
            image_out_tag="pca_dither2",
            center=[(5, 5), (5, 15), (15, 15), (15, 5)],
            cubes=1,
            size=0.2,
            gaussian=0.05,
            subframe=None,
            pca_number=1,
            mask_star=0.05,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("pca_dither2")

        data = self.pipeline.get_data("pca_dither2")
        assert np.sum(data) == pytest.approx(208.24417332523367, rel=1e-6, abs=0.0)
        assert data.shape == (20, 9, 9)

    def test_nodding_background(self) -> None:

        module = StackCubesModule(
            name_in="mean", image_in_tag="sky", image_out_tag="mean", combine="mean"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("mean")

        data = self.pipeline.get_data("mean")
        assert np.sum(data) == pytest.approx(
            21.108557759610548, rel=self.limit, abs=0.0
        )
        assert data.shape == (2, 11, 11)

        attr = self.pipeline.get_attribute("mean", "INDEX", static=False)
        assert np.sum(attr) == pytest.approx(1, rel=self.limit, abs=0.0)
        assert attr.shape == (2,)

        attr = self.pipeline.get_attribute("mean", "NFRAMES", static=False)
        assert np.sum(attr) == pytest.approx(2, rel=self.limit, abs=0.0)
        assert attr.shape == (2,)

        module = NoddingBackgroundModule(
            name_in="nodding",
            sky_in_tag="mean",
            science_in_tag="science",
            image_out_tag="nodding",
            mode="both",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("nodding")

        data = self.pipeline.get_data("nodding")
        assert np.sum(data) == pytest.approx(1.793466459074705, rel=self.limit, abs=0.0)
        assert data.shape == (10, 11, 11)

    def test_line_background_mean(self) -> None:

        module = LineSubtractionModule(
            name_in="line1",
            image_in_tag="science",
            image_out_tag="science_line1",
            combine="mean",
            mask=0.1,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("line1")

        data = self.pipeline.get_data("science_line1")
        assert np.sum(data) == pytest.approx(
            104.55443019231085, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_line_background_median(self) -> None:

        module = LineSubtractionModule(
            name_in="line2",
            image_in_tag="science",
            image_out_tag="science_line2",
            combine="median",
            mask=0.1,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("line2")

        data = self.pipeline.get_data("science_line2")
        assert np.sum(data) == pytest.approx(
            106.09825573198366, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)



import os

import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.hdf5reading import Hdf5ReadingModule
from pynpoint.readwrite.hdf5writing import Hdf5WritingModule
from pynpoint.util.tests import create_config, create_random, remove_test_data


class TestHdf5Writing:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_random(self.test_dir)
        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, files=["test.hdf5"])

    def test_hdf5_writing(self) -> None:

        module = Hdf5WritingModule(
            name_in="write1",
            file_name="test.hdf5",
            output_dir=None,
            tag_dictionary={"images": "data1"},
            keep_attributes=True,
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write1")

    def test_no_data_tag(self) -> None:

        module = Hdf5WritingModule(
            name_in="write2",
            file_name="test.hdf5",
            output_dir=None,
            tag_dictionary={"empty": "empty"},
            keep_attributes=True,
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("write2")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0]
            == "No data under the tag which is linked by the "
            "InputPort."
        )

    def test_overwrite_false(self) -> None:

        module = Hdf5WritingModule(
            name_in="write3",
            file_name="test.hdf5",
            output_dir=None,
            tag_dictionary={"images": "data2"},
            keep_attributes=True,
            overwrite=False,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write3")

    def test_dictionary_none(self) -> None:

        module = Hdf5WritingModule(
            name_in="write4",
            file_name="test.hdf5",
            output_dir=None,
            tag_dictionary=None,
            keep_attributes=True,
            overwrite=False,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write4")

    def test_hdf5_reading(self) -> None:

        module = Hdf5ReadingModule(
            name_in="read",
            input_filename="test.hdf5",
            input_dir=self.test_dir,
            tag_dictionary={"data1": "data1", "data2": "data2"},
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read")

        data1 = self.pipeline.get_data("data1")
        data2 = self.pipeline.get_data("data2")
        data3 = self.pipeline.get_data("images")
        assert data1 == pytest.approx(data2, rel=self.limit, abs=0.0)
        assert data2 == pytest.approx(data3, rel=self.limit, abs=0.0)

        attribute1 = self.pipeline.get_attribute("images", "PARANG", static=False)
        attribute2 = self.pipeline.get_attribute("data1", "PARANG", static=False)
        attribute3 = self.pipeline.get_attribute("data2", "PARANG", static=False)
        assert np.allclose(attribute1, attribute2, rtol=self.limit, atol=0.0)
        assert np.allclose(attribute2, attribute3, rtol=self.limit, atol=0.0)



import os

import h5py
import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.processing.darkflat import DarkCalibrationModule, FlatCalibrationModule
from pynpoint.util.tests import create_config, remove_test_data


class TestDarkFlat:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        np.random.seed(1)

        images = np.random.normal(loc=0, scale=2e-4, size=(5, 11, 11))
        dark = np.random.normal(loc=0, scale=2e-4, size=(5, 11, 11))
        flat = np.random.normal(loc=0, scale=2e-4, size=(5, 11, 11))
        crop = np.random.normal(loc=0, scale=2e-4, size=(5, 7, 7))

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "w") as hdf_file:
            hdf_file.create_dataset("images", data=images)
            hdf_file.create_dataset("dark", data=dark)
            hdf_file.create_dataset("flat", data=flat)
            hdf_file.create_dataset("crop", data=crop)

        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir)

    def test_input_data(self) -> None:

        data = self.pipeline.get_data("dark")
        assert np.sum(data) == pytest.approx(
            -2.0262345764957305e-05, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 11, 11)

        data = self.pipeline.get_data("flat")
        assert np.sum(data) == pytest.approx(
            0.0076413379497053, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 11, 11)

    def test_dark_calibration(self) -> None:

        module = DarkCalibrationModule(
            name_in="dark",
            image_in_tag="images",
            dark_in_tag="dark",
            image_out_tag="dark_cal",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("dark")

        data = self.pipeline.get_data("dark_cal")
        assert np.sum(data) == pytest.approx(
            0.00717386583629883, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 11, 11)

    def test_flat_calibration(self) -> None:

        module = FlatCalibrationModule(
            name_in="flat",
            image_in_tag="dark_cal",
            flat_in_tag="flat",
            image_out_tag="flat_cal",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("flat")

        data = self.pipeline.get_data("flat_cal")
        assert np.sum(data) == pytest.approx(
            0.00717439711853594, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 11, 11)

    def test_flat_crop(self) -> None:

        module = FlatCalibrationModule(
            name_in="flat_crop",
            image_in_tag="crop",
            flat_in_tag="flat",
            image_out_tag="flat_crop",
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("flat_crop")

        assert len(warning) == 1

        assert (
            warning[0].message.args[0]
            == "The calibration images were cropped around their "
            "center to match the shape of the science images."
        )

        data = self.pipeline.get_data("flat_crop")
        assert np.sum(data) == pytest.approx(
            -0.003242901413605404, rel=self.limit, abs=0.0
        )
        assert data.shape == (5, 7, 7)

    def test_flat_too_small(self) -> None:

        module = FlatCalibrationModule(
            name_in="flat_small",
            image_in_tag="flat",
            flat_in_tag="crop",
            image_out_tag="flat_small",
        )

        self.pipeline.add_module(module)

        with pytest.raises(ValueError) as error:
            self.pipeline.run_module("flat_small")

        assert (
            str(error.value) == "Shape of the calibration images is smaller than the "
            "science images."
        )



import os

import h5py
import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.processing.fluxposition import (
    FakePlanetModule,
    AperturePhotometryModule,
    FalsePositiveModule,
    SimplexMinimizationModule,
    MCMCsamplingModule,
    SystematicErrorModule,
)
from pynpoint.processing.stacksubset import DerotateAndStackModule
from pynpoint.processing.psfsubtraction import PcaPsfSubtractionModule
from pynpoint.util.tests import (
    create_config,
    create_star_data,
    create_fake_data,
    remove_test_data,
)


class TestFluxPosition:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_fake_data(self.test_dir + "adi")
        create_star_data(self.test_dir + "psf", npix=21, pos_star=10.0)
        create_star_data(self.test_dir + "ref", npix=21, pos_star=10.0)
        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, folders=["adi", "psf", "ref"])

    def test_read_data(self) -> None:

        module = FitsReadingModule(
            name_in="read1", image_tag="adi", input_dir=self.test_dir + "adi"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read1")

        data = self.pipeline.get_data("adi")
        assert np.sum(data) == pytest.approx(
            11.012854046962481, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 21, 21)

        self.pipeline.set_attribute(
            "adi", "PARANG", np.linspace(0.0, 180.0, 10), static=False
        )

        module = FitsReadingModule(
            name_in="read2", image_tag="psf", input_dir=self.test_dir + "psf"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read2")

        data = self.pipeline.get_data("psf")
        assert np.sum(data) == pytest.approx(
            108.43655133957289, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 21, 21)

        module = FitsReadingModule(
            name_in="read3", image_tag="ref", input_dir=self.test_dir + "psf"
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read3")

        data = self.pipeline.get_data("ref")
        assert np.sum(data) == pytest.approx(
            108.43655133957289, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 21, 21)

    def test_aperture_photometry(self) -> None:

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "a") as hdf_file:
            hdf_file["config"].attrs["CPU"] = 1

        module = AperturePhotometryModule(
            name_in="photometry1",
            image_in_tag="psf",
            phot_out_tag="photometry1",
            radius=0.1,
            position=None,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("photometry1")

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "a") as hdf_file:
            hdf_file["config"].attrs["CPU"] = 4

        module = AperturePhotometryModule(
            name_in="photometry2",
            image_in_tag="psf",
            phot_out_tag="photometry2",
            radius=0.1,
            position=None,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("photometry2")

        data = self.pipeline.get_data("photometry1")
        assert np.sum(data) == pytest.approx(
            100.80648929590365, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 1)

        data_multi = self.pipeline.get_data("photometry2")
        assert data.shape == data_multi.shape
        assert data == pytest.approx(data_multi, rel=self.limit, abs=0.0)

    def test_aperture_photometry_position(self) -> None:

        module = AperturePhotometryModule(
            name_in="photometry3",
            image_in_tag="psf",
            phot_out_tag="photometry3",
            radius=0.1,
            position=(10.0, 10.0),
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("photometry3")

        data = self.pipeline.get_data("photometry3")
        assert np.sum(data) == pytest.approx(
            100.80648929590365, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 1)

    def test_fake_planet(self) -> None:

        module = FakePlanetModule(
            position=(0.2, 180.0),
            magnitude=2.5,
            psf_scaling=1.0,
            interpolation="spline",
            name_in="fake",
            image_in_tag="adi",
            psf_in_tag="psf",
            image_out_tag="fake",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("fake")

        data = self.pipeline.get_data("fake")
        assert np.sum(data) == pytest.approx(
            21.273233520675586, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 21, 21)

    def test_psf_subtraction(self) -> None:

        module = PcaPsfSubtractionModule(
            pca_numbers=[
                1,
            ],
            name_in="pca",
            images_in_tag="fake",
            reference_in_tag="fake",
            res_mean_tag="res_mean",
            extra_rot=0.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("pca")

        data = self.pipeline.get_data("res_mean")
        assert np.sum(data) == pytest.approx(
            0.013659056187572433, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 21, 21)

    def test_false_positive(self) -> None:

        module = FalsePositiveModule(
            position=(10.0, 2.0),
            aperture=0.06,
            ignore=True,
            name_in="false1",
            image_in_tag="res_mean",
            snr_out_tag="snr_fpf1",
            optimize=False,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("false1")

        data = self.pipeline.get_data("snr_fpf1")
        assert data[0, 1] == pytest.approx(2.0, rel=self.limit, abs=0.0)
        assert data[0, 2] == pytest.approx(0.216, rel=self.limit, abs=0.0)
        assert data[0, 3] == pytest.approx(180.0, rel=self.limit, abs=0.0)
        assert data[0, 4] == pytest.approx(26.661611583224417, rel=self.limit, abs=0.0)
        assert data[0, 5] == pytest.approx(
            1.3375156927387672e-08, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 6)

    def test_false_positive_optimize(self) -> None:

        module = FalsePositiveModule(
            position=(10.0, 2.0),
            aperture=0.06,
            ignore=True,
            name_in="false2",
            image_in_tag="res_mean",
            snr_out_tag="snr_fpf2",
            optimize=True,
            offset=0.1,
            tolerance=0.01,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("false2")

        data = self.pipeline.get_data("snr_fpf2")
        assert data[0, 1] == pytest.approx(2.0747802734374985, rel=self.limit, abs=0.0)
        assert data[0, 2] == pytest.approx(0.2139883890923524, rel=self.limit, abs=0.0)
        assert data[0, 3] == pytest.approx(179.52168877335356, rel=self.limit, abs=0.0)
        assert data[0, 4] == pytest.approx(27.457328210661814, rel=self.limit, abs=0.0)
        assert data[0, 5] == pytest.approx(
            1.0905578015907869e-08, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 6)

    def test_simplex_minimization_hessian(self) -> None:

        module = SimplexMinimizationModule(
            name_in="simplex1",
            image_in_tag="fake",
            psf_in_tag="psf",
            res_out_tag="simplex_res",
            flux_position_tag="flux_position",
            position=(10.0, 3.0),
            magnitude=2.5,
            psf_scaling=-1.0,
            merit="hessian",
            aperture=0.06,
            sigma=0.0,
            tolerance=0.1,
            pca_number=1,
            cent_size=0.06,
            edge_size=None,
            extra_rot=0.0,
            reference_in_tag=None,
            residuals="median",
            offset=1.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("simplex1")

        data = self.pipeline.get_data("simplex_res")

        assert np.sum(data) == pytest.approx(
            0.2781582369128238, rel=self.limit, abs=0.0
        )
        assert data.shape == (35, 21, 21)

        data = self.pipeline.get_data("flux_position")
        assert data[24, 0] == pytest.approx(9.931627229080938, rel=self.limit, abs=0.0)
        assert data[24, 1] == pytest.approx(2.6575231481481456, rel=self.limit, abs=0.0)
        assert data[24, 2] == pytest.approx(0.1982554700445013, rel=self.limit, abs=0.0)
        assert data[24, 3] == pytest.approx(179.46648003649148, rel=self.limit, abs=0.0)
        assert data[24, 4] == pytest.approx(2.5256451474622708, rel=self.limit, abs=0.0)
        assert data.shape == (35, 6)

    def test_simplex_minimization_reference(self) -> None:

        module = SimplexMinimizationModule(
            name_in="simplex2",
            image_in_tag="fake",
            psf_in_tag="psf",
            res_out_tag="simplex_res_ref",
            flux_position_tag="flux_position_ref",
            position=(10.0, 3.0),
            magnitude=2.5,
            psf_scaling=-1.0,
            merit="poisson",
            aperture=0.06,
            sigma=0.0,
            tolerance=0.1,
            pca_number=1,
            cent_size=0.06,
            edge_size=None,
            extra_rot=0.0,
            reference_in_tag="ref",
            residuals="mean",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("simplex2")

        data = self.pipeline.get_data("simplex_res_ref")
        assert np.sum(data) == pytest.approx(9.734993454838076, rel=self.limit, abs=0.0)
        assert data.shape == (28, 21, 21)

        data = self.pipeline.get_data("flux_position_ref")
        assert data[27, 0] == pytest.approx(10.049019964116436, rel=self.limit, abs=0.0)
        assert data[27, 1] == pytest.approx(2.6444836362361936, rel=self.limit, abs=0.0)
        assert data[27, 2] == pytest.approx(
            0.19860335205689572, rel=self.limit, abs=0.0
        )
        assert data[27, 3] == pytest.approx(180.38183525629643, rel=self.limit, abs=0.0)
        assert data[27, 4] == pytest.approx(2.5496922175196, rel=self.limit, abs=0.0)
        assert data.shape == (28, 6)

    def test_mcmc_sampling(self) -> None:

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "a") as hdf_file:
            hdf_file["config"].attrs["CPU"] = 4

        module = DerotateAndStackModule(
            name_in="stack",
            image_in_tag="psf",
            image_out_tag="psf_stack",
            derotate=False,
            stack="mean",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("stack")

        data = self.pipeline.get_data("psf_stack")
        assert np.sum(data) == pytest.approx(
            10.843655133957288, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 21, 21)

        module = MCMCsamplingModule(
            name_in="mcmc1",
            image_in_tag="adi",
            psf_in_tag="psf_stack",
            chain_out_tag="mcmc",
            param=(0.15, 0.0, 1.0),
            bounds=((0.1, 0.2), (-2.0, 2.0), (-1.0, 2.0)),
            nwalkers=6,
            nsteps=5,
            psf_scaling=-1.0,
            pca_number=1,
            aperture=(10, 16, 0.06),
            mask=None,
            extra_rot=0.0,
            merit="gaussian",
            residuals="median",
            resume=False,
            sigma=(1e-3, 1e-1, 1e-2),
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("mcmc1")

        data = self.pipeline.get_data("mcmc")
        assert data.shape == (5, 6, 3)

        data = self.pipeline.get_data("mcmc_backend")
        assert data.shape == (3,)

        module = MCMCsamplingModule(
            name_in="mcmc2",
            image_in_tag="adi",
            psf_in_tag="psf_stack",
            chain_out_tag="mcmc",
            param=(0.15, 0.0, 1.0),
            bounds=((0.1, 0.2), (-2.0, 2.0), (-1.0, 2.0)),
            nwalkers=6,
            nsteps=5,
            psf_scaling=-1.0,
            pca_number=1,
            aperture=(10, 16, 0.06),
            mask=None,
            extra_rot=0.0,
            merit="gaussian",
            residuals="median",
            resume=True,
            sigma=(1e-3, 1e-1, 1e-2),
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("mcmc2")

        data = self.pipeline.get_data("mcmc")
        assert data.shape == (10, 6, 3)

        data = self.pipeline.get_data("mcmc_backend")
        assert data.shape == (3,)

    def test_systematic_error(self) -> None:

        module = SystematicErrorModule(
            name_in="error",
            image_in_tag="adi",
            psf_in_tag="psf",
            offset_out_tag="offset",
            position=(0.162, 0.0),
            magnitude=5.0,
            angles=(0.0, 180.0, 2),
            psf_scaling=1.0,
            merit="gaussian",
            aperture=0.06,
            tolerance=0.1,
            pca_number=1,
            mask=(None, None),
            extra_rot=0.0,
            residuals="median",
            offset=1.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("error")

        data = self.pipeline.get_data("offset")
        assert data[0, 0] == pytest.approx(
            -0.001114020093541973, rel=self.limit, abs=0.0
        )
        assert data[0, 1] == pytest.approx(
            -0.012163271644183737, rel=self.limit, abs=0.0
        )
        assert data[0, 2] == pytest.approx(
            -0.017943854263249293, rel=self.limit, abs=0.0
        )
        assert data[0, 3] == pytest.approx(
            0.001282493868968615, rel=self.limit, abs=0.0
        )
        assert data[0, 4] == pytest.approx(
            -0.04125986733475884, rel=self.limit, abs=0.0
        )
        assert data.shape == (2, 5)



import os

import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.attr_reading import (
    ParangReadingModule,
    AttributeReadingModule,
    WavelengthReadingModule,
)
from pynpoint.util.tests import create_config, create_random, remove_test_data


class TestAttributeReading:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_random(self.test_dir, nimages=10)
        create_config(self.test_dir + "PynPoint_config.ini")

        np.savetxt(self.test_dir + "parang.dat", np.arange(10.0, 20.0, 1.0))
        np.savetxt(self.test_dir + "new.dat", np.arange(20.0, 30.0, 1.0))
        np.savetxt(self.test_dir + "attribute.dat", np.arange(0, 10, 1), fmt="%i")
        np.savetxt(self.test_dir + "wavelength.dat", np.arange(0.0, 10.0, 1.0))

        data2d = np.random.normal(loc=0, scale=2e-4, size=(10, 10))
        np.savetxt(self.test_dir + "data_2d.dat", data2d)

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(
            self.test_dir,
            files=[
                "parang.dat",
                "new.dat",
                "attribute.dat",
                "data_2d.dat",
                "wavelength.dat",
            ],
        )

    def test_input_data(self) -> None:

        data = self.pipeline.get_data("images")
        assert np.sum(data) == pytest.approx(
            0.007133341144768919, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_parang_reading(self) -> None:

        module = ParangReadingModule(
            name_in="parang1",
            data_tag="images",
            file_name="parang.dat",
            input_dir=None,
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("parang1")

        data = self.pipeline.get_data("header_images/PARANG")
        assert data.dtype == "float64"
        assert data == pytest.approx(
            np.arange(10.0, 20.0, 1.0), rel=self.limit, abs=0.0
        )
        assert data.shape == (10,)

    def test_parang_reading_same(self) -> None:

        module = ParangReadingModule(
            name_in="parang2",
            data_tag="images",
            file_name="parang.dat",
            input_dir=None,
            overwrite=True,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("parang2")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0] == "The PARANG attribute is already present and "
            "contains the same values as are present in "
            "parang.dat."
        )

    def test_parang_reading_present(self) -> None:

        module = ParangReadingModule(
            name_in="parang3",
            data_tag="images",
            file_name="new.dat",
            input_dir=None,
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("parang3")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0]
            == "The PARANG attribute is already present. Set the "
            "'overwrite' parameter to True in order to "
            "overwrite the values with new.dat."
        )

    def test_parang_reading_overwrite(self) -> None:

        module = ParangReadingModule(
            file_name="new.dat",
            name_in="parang4",
            input_dir=None,
            data_tag="images",
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("parang4")

    def test_parang_reading_2d(self) -> None:

        module = ParangReadingModule(
            name_in="parang6",
            data_tag="images",
            file_name="data_2d.dat",
            input_dir=None,
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.raises(ValueError) as error:
            self.pipeline.run_module("parang6")

        assert (
            str(error.value)
            == "The input file data_2d.dat should contain a 1D data set with "
            "the parallactic angles."
        )

    def test_attribute_reading(self) -> None:

        module = AttributeReadingModule(
            file_name="attribute.dat",
            attribute="EXP_NO",
            name_in="attribute1",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("attribute1")

        data = self.pipeline.get_data("header_images/EXP_NO")
        assert data.dtype == "int64"
        assert data == pytest.approx(np.arange(10), rel=self.limit, abs=0.0)
        assert data.shape == (10,)

    def test_attribute_reading_present(self) -> None:

        module = AttributeReadingModule(
            file_name="parang.dat",
            attribute="PARANG",
            name_in="attribute3",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("attribute3")

        assert (
            warning[0].message.args[0]
            == "The attribute 'PARANG' is already present. Set "
            "the 'overwrite' parameter to True in order to "
            "overwrite the values with parang.dat."
        )

    def test_attribute_reading_invalid(self) -> None:

        module = AttributeReadingModule(
            file_name="attribute.dat",
            attribute="test",
            name_in="attribute4",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.raises(ValueError) as error:
            self.pipeline.run_module("attribute4")

        assert str(error.value) == "'test' is not a valid attribute."

    def test_attribute_reading_2d(self) -> None:

        module = AttributeReadingModule(
            file_name="data_2d.dat",
            attribute="DITHER_X",
            name_in="attribute5",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.raises(ValueError) as error:
            self.pipeline.run_module("attribute5")

        assert (
            str(error.value)
            == "The input file data_2d.dat should contain a 1D list with "
            "attributes."
        )

    def test_attribute_reading_same(self) -> None:

        module = AttributeReadingModule(
            file_name="attribute.dat",
            attribute="EXP_NO",
            name_in="attribute6",
            input_dir=None,
            data_tag="images",
            overwrite=True,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("attribute6")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0]
            == "The 'EXP_NO' attribute is already present and "
            "contains the same values as are present in "
            "attribute.dat."
        )

    def test_attribute_reading_overwrite(self) -> None:

        module = AttributeReadingModule(
            file_name="parang.dat",
            attribute="PARANG",
            name_in="attribute7",
            input_dir=None,
            data_tag="images",
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("attribute7")

        data = self.pipeline.get_attribute("images", "PARANG", static=False)
        assert data == pytest.approx(
            np.arange(10.0, 20.0, 1.0), rel=self.limit, abs=0.0
        )

    def test_wavelength_reading(self) -> None:

        module = WavelengthReadingModule(
            file_name="wavelength.dat",
            name_in="wavelength1",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("wavelength1")

        data = self.pipeline.get_data("header_images/WAVELENGTH")
        assert data.dtype == "float64"
        assert data == pytest.approx(np.arange(10.0), rel=self.limit, abs=0.0)
        assert data.shape == (10,)

    def test_wavelength_reading_same(self) -> None:

        module = WavelengthReadingModule(
            file_name="wavelength.dat",
            name_in="wavelength2",
            input_dir=None,
            data_tag="images",
            overwrite=True,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("wavelength2")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0]
            == "The WAVELENGTH attribute is already present and "
            "contains the same values as are present in "
            "wavelength.dat."
        )

    def test_wavelength_reading_present(self) -> None:

        module = WavelengthReadingModule(
            file_name="new.dat",
            name_in="wavelength3",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("wavelength3")

        assert len(warning) == 1
        assert (
            warning[0].message.args[0]
            == "The WAVELENGTH attribute is already present. Set "
            "the 'overwrite' parameter to True in order to "
            "overwrite the values with new.dat."
        )

    def test_wavelength_reading_overwrite(self) -> None:

        module = WavelengthReadingModule(
            file_name="new.dat",
            name_in="wavelength4",
            input_dir=None,
            data_tag="images",
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("wavelength4")

    def test_wavelength_reading_2d(self) -> None:

        module = WavelengthReadingModule(
            file_name="data_2d.dat",
            name_in="wavelength6",
            input_dir=None,
            data_tag="images",
            overwrite=False,
        )

        self.pipeline.add_module(module)

        with pytest.raises(ValueError) as error:
            self.pipeline.run_module("wavelength6")

        assert (
            str(error.value)
            == "The input file data_2d.dat should contain a 1D data set with "
            "the wavelengths."
        )



import os

import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.processing.stacksubset import (
    StackAndSubsetModule,
    StackCubesModule,
    DerotateAndStackModule,
    CombineTagsModule,
)
from pynpoint.util.tests import (
    create_config,
    create_star_data,
    create_ifs_data,
    remove_test_data,
)


class TestStackSubset:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_ifs_data(self.test_dir + "data_ifs")
        create_star_data(self.test_dir + "data")
        create_star_data(self.test_dir + "extra")

        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, folders=["data_ifs", "extra", "data"])

    def test_read_data(self) -> None:

        module = FitsReadingModule(
            name_in="read1",
            image_tag="images",
            input_dir=self.test_dir + "data",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read1")

        data = self.pipeline.get_data("images")
        assert np.mean(data) == pytest.approx(
            0.08722544528764692, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

        module = FitsReadingModule(
            name_in="read2",
            image_tag="extra",
            input_dir=self.test_dir + "extra",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read2")

        extra = self.pipeline.get_data("extra")
        assert data == pytest.approx(extra, rel=self.limit, abs=0.0)

        module = FitsReadingModule(
            name_in="read_ifs",
            image_tag="images_ifs",
            input_dir=self.test_dir + "data_ifs",
            overwrite=True,
            check=True,
            ifs_data=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read_ifs")
        self.pipeline.set_attribute(
            "images_ifs", "PARANG", np.linspace(0.0, 180.0, 10), static=False
        )

        data = self.pipeline.get_data("images_ifs")
        assert np.sum(data) == pytest.approx(749.8396528807369, rel=self.limit, abs=0.0)
        assert data.shape == (3, 10, 21, 21)

    def test_stack_and_subset(self) -> None:

        self.pipeline.set_attribute("images", "PARANG", np.arange(10.0), static=False)

        module = StackAndSubsetModule(
            name_in="stack1",
            image_in_tag="images",
            image_out_tag="stack1",
            random=4,
            stacking=2,
            combine="mean",
            max_rotation=None,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("stack1")

        data = self.pipeline.get_data("stack1")
        assert np.mean(data) == pytest.approx(
            0.08758276283743936, rel=self.limit, abs=0.0
        )
        assert data.shape == (4, 11, 11)

        data = self.pipeline.get_data("header_stack1/INDEX")
        assert data == pytest.approx(np.arange(4), rel=self.limit, abs=0.0)
        assert data.shape == (4,)

        data = self.pipeline.get_data("header_stack1/PARANG")
        assert data == pytest.approx([0.5, 2.5, 6.5, 8.5], rel=self.limit, abs=0.0)
        assert data.shape == (4,)

    def test_stack_max_rotation(self) -> None:

        angles = np.arange(10.0)
        angles[1:6] = 3.0
        angles[9] = 50.0

        self.pipeline.set_attribute("images", "PARANG", angles, static=False)

        module = StackAndSubsetModule(
            name_in="stack2",
            image_in_tag="images",
            image_out_tag="stack2",
            random=None,
            stacking=2,
            combine="median",
            max_rotation=1.0,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("stack2")

        assert len(warning) == 1

        assert (
            warning[0].message.args[0]
            == "Testing of util.module.stack_angles has been "
            "limited, please use carefully."
        )

        data = self.pipeline.get_data("stack2")
        assert np.mean(data) == pytest.approx(
            0.08580759396987508, rel=self.limit, abs=0.0
        )
        assert data.shape == (7, 11, 11)

        data = self.pipeline.get_data("header_stack2/INDEX")
        assert data == pytest.approx(np.arange(7), rel=self.limit, abs=0.0)
        assert data.shape == (7,)

        data = self.pipeline.get_data("header_stack2/PARANG")
        assert data.shape == (7,)

        self.pipeline.set_attribute("images", "PARANG", np.arange(10.0), static=False)

    def test_stack_cube(self) -> None:

        module = StackCubesModule(
            name_in="stackcube",
            image_in_tag="images",
            image_out_tag="mean",
            combine="mean",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("stackcube")

        data = self.pipeline.get_data("mean")
        assert np.mean(data) == pytest.approx(
            0.08722544528764689, rel=self.limit, abs=0.0
        )
        assert data.shape == (2, 11, 11)

        attribute = self.pipeline.get_attribute("mean", "INDEX", static=False)
        assert np.mean(attribute) == pytest.approx(0.5, rel=self.limit, abs=0.0)
        assert attribute.shape == (2,)

        attribute = self.pipeline.get_attribute("mean", "NFRAMES", static=False)
        assert np.mean(attribute) == pytest.approx(1, rel=self.limit, abs=0.0)
        assert attribute.shape == (2,)

    def test_derotate_and_stack(self) -> None:

        module = DerotateAndStackModule(
            name_in="derotate1",
            image_in_tag="images",
            image_out_tag="derotate1",
            derotate=True,
            stack="mean",
            extra_rot=10.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("derotate1")

        data = self.pipeline.get_data("derotate1")
        assert np.mean(data) == pytest.approx(
            0.08709860116308817, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 11, 11)

        module = DerotateAndStackModule(
            name_in="derotate2",
            image_in_tag="images",
            image_out_tag="derotate2",
            derotate=False,
            stack="median",
            extra_rot=0.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("derotate2")

        data = self.pipeline.get_data("derotate2")
        assert np.mean(data) == pytest.approx(
            0.0861160094566323, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 11, 11)

        data = self.pipeline.get_data("derotate2")
        assert np.mean(data) == pytest.approx(
            0.0861160094566323, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 11, 11)

        module = DerotateAndStackModule(
            name_in="derotate_ifs1",
            image_in_tag="images_ifs",
            image_out_tag="derotate_ifs1",
            derotate=True,
            stack="mean",
            extra_rot=0.0,
            dimension="time",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("derotate_ifs1")

        data = self.pipeline.get_data("derotate_ifs1")
        assert np.mean(data) == pytest.approx(
            0.1884438996655355, rel=self.limit, abs=0.0
        )
        assert data.shape == (3, 1, 21, 21)

        module = DerotateAndStackModule(
            name_in="derotate_ifs2",
            image_in_tag="images_ifs",
            image_out_tag="derotate_ifs2",
            derotate=False,
            stack="median",
            extra_rot=0.0,
            dimension="wavelength",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("derotate_ifs2")

        data = self.pipeline.get_data("derotate_ifs2")
        assert np.mean(data) == pytest.approx(
            0.055939644983170146, rel=self.limit, abs=0.0
        )
        assert data.shape == (1, 10, 21, 21)

        module = DerotateAndStackModule(
            name_in="derotate_ifs3",
            image_in_tag="images_ifs",
            image_out_tag="derotate_ifs3",
            derotate=True,
            stack=None,
            extra_rot=0.0,
            dimension="wavelength",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("derotate_ifs3")

        data = self.pipeline.get_data("derotate_ifs3")
        assert np.mean(data) == pytest.approx(
            0.05653316989966066, rel=self.limit, abs=0.0
        )
        assert data.shape == (3, 10, 21, 21)

    def test_combine_tags(self) -> None:

        module = CombineTagsModule(
            image_in_tags=["images", "extra"],
            check_attr=True,
            index_init=False,
            name_in="combine1",
            image_out_tag="combine1",
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("combine1")

        assert len(warning) == 1

        assert (
            warning[0].message.args[0]
            == "The non-static keyword FILES is already used but "
            "with different values. It is advisable to only "
            "combine tags that descend from the same data set."
        )

        data = self.pipeline.get_data("combine1")
        assert np.mean(data) == pytest.approx(
            0.0872254452876469, rel=self.limit, abs=0.0
        )
        assert data.shape == (20, 11, 11)

        data = self.pipeline.get_data("header_combine1/INDEX")
        assert data[19] == 9
        assert data.shape == (20,)

        module = CombineTagsModule(
            image_in_tags=["images", "extra"],
            check_attr=False,
            index_init=True,
            name_in="combine2",
            image_out_tag="combine2",
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("combine2")

        data = self.pipeline.get_data("combine1")
        extra = self.pipeline.get_data("combine2")
        assert data == pytest.approx(extra, rel=self.limit, abs=0.0)

        data = self.pipeline.get_data("header_combine2/INDEX")
        assert data[19] == 19
        assert data.shape == (20,)



import os

import h5py
import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.processing.extract import StarExtractionModule, ExtractBinaryModule
from pynpoint.util.tests import (
    create_config,
    create_star_data,
    create_fake_data,
    remove_test_data,
)


class TestExtract:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_star_data(self.test_dir + "star")
        create_fake_data(self.test_dir + "binary")
        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(path=self.test_dir, folders=["star", "binary"])

    def test_read_data(self) -> None:

        module = FitsReadingModule(
            name_in="read1",
            image_tag="star",
            input_dir=self.test_dir + "star",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read1")

        data = self.pipeline.get_data("star")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

        module = FitsReadingModule(
            name_in="read2",
            image_tag="binary",
            input_dir=self.test_dir + "binary",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read2")

        data = self.pipeline.get_data("binary")
        assert np.sum(data) == pytest.approx(
            11.012854046962481, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 21, 21)

        self.pipeline.set_attribute(
            "binary", "PARANG", -1.0 * np.linspace(0.0, 180.0, 10), static=False
        )

    def test_extract_position_none(self) -> None:

        module = StarExtractionModule(
            name_in="extract1",
            image_in_tag="star",
            image_out_tag="extract1",
            index_out_tag="index",
            image_size=0.2,
            fwhm_star=0.1,
            position=None,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("extract1")

        assert len(warning) == 3

        assert (
            warning[0].message.args[0]
            == "Can not store the attribute 'INSTRUMENT' because "
            "the dataset 'index' does not exist."
        )

        assert (
            warning[1].message.args[0]
            == "Can not store the attribute 'PIXSCALE' because "
            "the dataset 'index' does not exist."
        )

        assert (
            warning[2].message.args[0] == "Can not store the attribute 'History: "
            "StarExtractionModule' because the dataset "
            "'index' does not exist."
        )

        data = self.pipeline.get_data("extract1")
        assert np.sum(data) == pytest.approx(
            104.93318507061295, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 9, 9)

    def test_extract_center_none(self) -> None:

        module = StarExtractionModule(
            name_in="extract2",
            image_in_tag="star",
            image_out_tag="extract2",
            index_out_tag="index",
            image_size=0.2,
            fwhm_star=0.1,
            position=(None, None, 0.2),
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("extract2")

        assert len(warning) == 3

        assert (
            warning[0].message.args[0]
            == "Can not store the attribute 'INSTRUMENT' because "
            "the dataset 'index' does not exist."
        )

        assert (
            warning[1].message.args[0]
            == "Can not store the attribute 'PIXSCALE' because "
            "the dataset 'index' does not exist."
        )

        assert (
            warning[2].message.args[0] == "Can not store the attribute 'History: "
            "StarExtractionModule' because the dataset "
            "'index' does not exist."
        )

        data = self.pipeline.get_data("extract2")
        assert np.sum(data) == pytest.approx(
            104.93318507061295, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 9, 9)

    def test_extract_position(self) -> None:

        module = StarExtractionModule(
            name_in="extract7",
            image_in_tag="star",
            image_out_tag="extract7",
            index_out_tag=None,
            image_size=0.2,
            fwhm_star=0.1,
            position=(5, 5, 0.2),
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("extract7")

        data = self.pipeline.get_data("extract7")
        assert np.sum(data) == pytest.approx(
            104.93318507061295, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 9, 9)

    def test_extract_too_large(self) -> None:

        module = StarExtractionModule(
            name_in="extract3",
            image_in_tag="star",
            image_out_tag="extract3",
            index_out_tag=None,
            image_size=0.2,
            fwhm_star=0.1,
            position=(2, 2, 0.05),
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("extract3")

        assert len(warning) == 10

        assert (
            warning[0].message.args[0]
            == f"Chosen image size is too large to crop the image "
            f"around the brightest pixel (image index = 0, "
            f"pixel [x, y] = [2, 2]). Using the center of "
            f"the image instead."
        )

        data = self.pipeline.get_data("extract3")
        assert np.sum(data) == pytest.approx(
            104.93318507061295, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 9, 9)

    def test_star_extract_cpu(self) -> None:

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "a") as hdf_file:
            hdf_file["config"].attrs["CPU"] = 4

        module = StarExtractionModule(
            name_in="extract4",
            image_in_tag="star",
            image_out_tag="extract4",
            index_out_tag="index",
            image_size=0.2,
            fwhm_star=0.1,
            position=(2, 2, 0.05),
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("extract4")

        # assert len(warning) == 8

        assert (
            warning[0].message.args[0] == "The 'index_out_port' can only be used if "
            "CPU = 1. No data will be stored to this output port."
        )

        assert (
            warning[1].message.args[0]
            == "Chosen image size is too large to crop the image "
            "around the brightest pixel (image index = 0, "
            "pixel [x, y] = [2, 2]). Using the center of the "
            "image instead."
        )

    def test_extract_binary(self) -> None:

        with h5py.File(self.test_dir + "PynPoint_database.hdf5", "a") as hdf_file:
            hdf_file["config"].attrs["CPU"] = 1

        module = ExtractBinaryModule(
            pos_center=(10.0, 10.0),
            pos_binary=(10.0, 16.0),
            name_in="extract5",
            image_in_tag="binary",
            image_out_tag="extract5",
            image_size=0.15,
            search_size=0.07,
            filter_size=None,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("extract5")

        data = self.pipeline.get_data("extract5")
        assert np.sum(data) == pytest.approx(
            1.3419098759577548, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 7, 7)

    def test_extract_binary_filter(self) -> None:

        module = ExtractBinaryModule(
            pos_center=(10.0, 10.0),
            pos_binary=(10.0, 16.0),
            name_in="extract6",
            image_in_tag="binary",
            image_out_tag="extract6",
            image_size=0.15,
            search_size=0.07,
            filter_size=0.05,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("extract6")

        data = self.pipeline.get_data("extract6")
        assert np.sum(data) == pytest.approx(
            1.3789593661036972, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 7, 7)



import os

import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.processing.basic import (
    SubtractImagesModule,
    AddImagesModule,
    RotateImagesModule,
    RepeatImagesModule,
)
from pynpoint.util.tests import create_config, remove_test_data, create_star_data


class TestBasic:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_star_data(self.test_dir + "data1")
        create_star_data(self.test_dir + "data2")
        create_star_data(self.test_dir + "data3")

        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, folders=["data1", "data2", "data3"])

    def test_read_data(self) -> None:

        module = FitsReadingModule(
            name_in="read1",
            image_tag="data1",
            input_dir=self.test_dir + "data1",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)

        module = FitsReadingModule(
            name_in="read2",
            image_tag="data2",
            input_dir=self.test_dir + "data2",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)

        module = FitsReadingModule(
            name_in="read3",
            image_tag="data3",
            input_dir=self.test_dir + "data3",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)

        self.pipeline.run_module("read1")
        self.pipeline.run_module("read2")
        self.pipeline.run_module("read3")

        data = self.pipeline.get_data("data1")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

        data = self.pipeline.get_data("data2")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

        data = self.pipeline.get_data("data3")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_subtract_images(self) -> None:

        module = SubtractImagesModule(
            name_in="subtract",
            image_in_tags=("data1", "data2"),
            image_out_tag="subtract",
            scaling=1.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("subtract")

        data = self.pipeline.get_data("subtract")
        assert np.sum(data) == pytest.approx(0.0, rel=self.limit, abs=0.0)
        assert data.shape == (10, 11, 11)

    def test_add_images(self) -> None:

        module = AddImagesModule(
            name_in="add",
            image_in_tags=("data1", "data2"),
            image_out_tag="add",
            scaling=1.0,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("add")

        data = self.pipeline.get_data("add")
        assert np.sum(data) == pytest.approx(
            211.08557759610554, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_rotate_images(self) -> None:

        module = RotateImagesModule(
            name_in="rotate", image_in_tag="data1", image_out_tag="rotate", angle=10.0
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("rotate")

        data = self.pipeline.get_data("rotate")
        assert np.sum(data) == pytest.approx(
            105.86657256219851, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_repeat_images(self) -> None:

        module = RepeatImagesModule(
            name_in="repeat", image_in_tag="data1", image_out_tag="repeat", repeat=2
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("repeat")

        data1 = self.pipeline.get_data("data1")
        assert data1.shape == (10, 11, 11)

        data2 = self.pipeline.get_data("repeat")
        assert data2.shape == (20, 11, 11)

        assert data1 == pytest.approx(data2[0:10,], rel=self.limit, abs=0.0)
        assert data1 == pytest.approx(data2[10:20,], rel=self.limit, abs=0.0)



import os

import pytest
import numpy as np

from pynpoint.core.pypeline import Pypeline
from pynpoint.readwrite.fitsreading import FitsReadingModule
from pynpoint.readwrite.fitswriting import FitsWritingModule
from pynpoint.util.tests import create_config, create_star_data, remove_test_data


class TestFitsWriting:

    def setup_class(self) -> None:

        self.limit = 1e-10
        self.test_dir = os.path.dirname(__file__) + "/"

        create_star_data(self.test_dir + "fits_data")
        create_config(self.test_dir + "PynPoint_config.ini")

        self.pipeline = Pypeline(self.test_dir, self.test_dir, self.test_dir)

    def teardown_class(self) -> None:

        remove_test_data(self.test_dir, folders=["fits_data"], files=["test.fits"])

    def test_fits_reading(self) -> None:

        module = FitsReadingModule(
            name_in="read",
            input_dir=self.test_dir + "fits_data",
            image_tag="images",
            overwrite=True,
            check=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("read")

        data = self.pipeline.get_data("images")
        assert np.sum(data) == pytest.approx(
            105.54278879805277, rel=self.limit, abs=0.0
        )
        assert data.shape == (10, 11, 11)

    def test_fits_writing(self) -> None:

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write1",
            output_dir=None,
            data_tag="images",
            data_range=None,
            overwrite=True,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write1")

    def test_filename_extension(self) -> None:

        with pytest.raises(ValueError) as error:
            FitsWritingModule(
                file_name="test.dat",
                name_in="write3",
                output_dir=None,
                data_tag="images",
                data_range=None,
                overwrite=True,
                subset_size=None,
            )

        assert str(error.value) == "Output 'file_name' requires the FITS extension."

    def test_data_range(self) -> None:

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write4",
            output_dir=None,
            data_tag="images",
            data_range=(0, 10),
            overwrite=True,
            subset_size=None,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write4")

    def test_not_overwritten(self) -> None:

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write5",
            output_dir=None,
            data_tag="images",
            data_range=None,
            overwrite=False,
            subset_size=None,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("write5")

        assert len(warning) == 1

        assert (
            warning[0].message.args[0]
            == "Filename already present. Use overwrite=True "
            "to overwrite an existing FITS file."
        )

    def test_subset_size(self) -> None:

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write6",
            output_dir=None,
            data_tag="images",
            data_range=None,
            overwrite=True,
            subset_size=10,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write6")

    def test_subset_size_data_range(self) -> None:

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write7",
            output_dir=None,
            data_tag="images",
            data_range=(8, 18),
            overwrite=True,
            subset_size=10,
        )

        self.pipeline.add_module(module)
        self.pipeline.run_module("write7")

    def test_attribute_length(self) -> None:

        text = "long_text_long_text_long_text_long_text_long_text_long_text_long_text_long_text"

        self.pipeline.set_attribute("images", "short", "value", static=True)
        self.pipeline.set_attribute(
            "images", "longer_than_eight1", "value", static=True
        )
        self.pipeline.set_attribute("images", "longer_than_eight2", text, static=True)

        module = FitsWritingModule(
            file_name="test.fits",
            name_in="write8",
            output_dir=None,
            data_tag="images",
            data_range=None,
            overwrite=True,
            subset_size=None,
        )

        self.pipeline.add_module(module)

        with pytest.warns(UserWarning) as warning:
            self.pipeline.run_module("write8")

        assert len(warning) == 1

        assert (
            warning[0].message.args[0]
            == "Key 'hierarch longer_than_eight2' with value "
            "'long_text_long_text_long_text_long_text_long_"
            "text_long_text_long_text_long_text' is too "
            "long for the FITS format. To avoid an error, "
            "the value was truncated to 'long_text_long_text"
            "_long_text_long_text_long_tex'."
        )
